@@ -13,11 +13,35 @@ module Text.Lucius
       -- ** Runtime
     , luciusRT
     , luciusRT'
-      -- * Re-export cassius
-    , module Text.Cassius
+    , -- * Datatypes
+      Css
+    , CssUrl
+      -- * Type class
+    , ToCss (..)
+      -- * Rendering
+    , renderCss
+    , renderCssUrl
+      -- * ToCss instances
+      -- ** Color
+    , Color (..)
+    , colorRed
+    , colorBlack
+      -- ** Size
+    , mkSize
+    , AbsoluteUnit (..)
+    , AbsoluteSize (..)
+    , absoluteSize
+    , EmSize (..)
+    , ExSize (..)
+    , PercentageSize (..)
+    , percentageSize
+    , PixelSize (..)
+      -- * Internal
+    , parseTopLevels
+    , luciusUsedIdentifiers
     ) where
 
-import Text.Cassius hiding (cassius, cassiusFile, cassiusFileDebug, cassiusFileReload)
+import Text.CssCommon
 import Text.Shakespeare.Base
 import Language.Haskell.TH.Quote (QuasiQuoter (..))
 import Language.Haskell.TH.Syntax
@@ -28,9 +52,10 @@ import Text.Css
 import Data.Char (isSpace, toLower, toUpper)
 import Numeric (readHex)
 import Control.Applicative ((<$>))
-import Control.Monad (when)
+import Control.Monad (when, unless)
 import Data.Either (partitionEithers)
 import Data.Monoid (mconcat)
+import Data.List (isSuffixOf)
 
 -- |
 --
@@ -160,7 +185,7 @@ luciusFile fp = do
     luciusFromString contents
 
 luciusFileDebug, luciusFileReload :: FilePath -> Q Exp
-luciusFileDebug = cssFileDebug [|parseTopLevels|] parseTopLevels
+luciusFileDebug = cssFileDebug False [|parseTopLevels|] parseTopLevels
 luciusFileReload = luciusFileDebug
 
 parseTopLevels :: Parser [TopLevel]
@@ -172,7 +197,7 @@ parseTopLevels =
             ignore = many (whiteSpace1 <|> string' "<!--" <|> string' "-->")
                         >> return ()
         ignore
-        tl <- ((charset <|> media <|> impor <|> var <|> fmap TopBlock parseBlock) >>= \x -> go (front . (:) x))
+        tl <- ((charset <|> media <|> impor <|> topAtBlock <|> var <|> fmap TopBlock parseBlock) >>= \x -> go (front . (:) x))
             <|> (return $ map compressTopLevel $ front [])
         ignore
         return tl
@@ -204,6 +229,17 @@ parseTopLevels =
         _ <- char ';'
         let trimS = reverse . dropWhile isSpace . reverse . dropWhile isSpace
         return $ TopVar (trimS k) (trimS v)
+    topAtBlock = do
+        (name, selector) <- try $ do
+            _ <- char '@'
+            name <- many1 $ noneOf " \t"
+            _ <- many1 $ oneOf " \t"
+            unless ("keyframes" `isSuffixOf` name) $ fail "only accepting keyframes"
+            selector <- parseContents "{"
+            _ <- char '{'
+            return (name, selector)
+        b <- parseBlocks id
+        return $ TopAtBlock name selector b
     parseBlocks front = do
         whiteSpace
         (char '}' >> return (map compressBlock $ front []))
@@ -250,3 +286,8 @@ luciusRT' tl =
 
 luciusRT :: TL.Text -> [(Text, Text)] -> Either String TL.Text
 luciusRT tl scope = either Left (Right . renderCss . CssWhitespace) $ either Left ($ scope) (luciusRT' tl)
+
+-- | Determine which identifiers are used by the given template, useful for
+-- creating systems like yesod devel.
+luciusUsedIdentifiers :: String -> [(Deref, VarType)]
+luciusUsedIdentifiers = cssUsedIdentifiers False parseTopLevels
